﻿// Sample Code is provided for the purpose of illustration only and is not intended to be used in a production environment. 
// THIS SAMPLE CODE AND ANY RELATED INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
// EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY 
// AND/OR FITNESS FOR A PARTICULAR PURPOSE

namespace HardwareApiCSharpSample
{
    using System;
    using System.Collections.Generic;
    using System.Globalization;
    using System.Net.Http;
    using System.Threading.Tasks;
    using HardwareApiCSharpSample.Models;
    using Newtonsoft.Json.Linq;

    /// <summary>
    /// This sample update does a full submission update, updating listings info, images, and packages
    /// </summary>
    public class ProductSubmissionUpdateSample
    {
        private ClientConfiguration _clientConfiguration;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="c">An instance of ClientConfiguration that contains all parameters populated</param>
        public ProductSubmissionUpdateSample(ClientConfiguration clientConfiguration)
        {
            this._clientConfiguration = clientConfiguration;
        }

        public void RunProductSubmissionUpdateSample()
        {
            // **********************
            //       SETTINGS
            // **********************
            var productId = this._clientConfiguration.ProductId;
            var submissionId = this._clientConfiguration.SubmissionId;
            var clientId = this._clientConfiguration.ClientId;
            var clientSecret = this._clientConfiguration.ClientSecret;
            var serviceEndpoint = this._clientConfiguration.ServiceUrl;
            var tokenEndpoint = this._clientConfiguration.TokenEndpoint;

            // Get authorization token
            Console.WriteLine("Getting authorization token ");
            var accessToken = HardwareIngestionClient.GetClientCredentialAccessTokenAsync(
                tokenEndpoint,
                clientId,
                clientSecret).Result;

            Console.WriteLine("Getting product ");
            var client = new HardwareIngestionClient(accessToken, serviceEndpoint);
            dynamic product = client.Invoke<dynamic>(
                HttpMethod.Get,
                relativeUrl: string.Format(
                    CultureInfo.InvariantCulture,
                    HardwareIngestionClient.GetProductUrlTemplate,
                    HardwareIngestionClient.Version,
                    HardwareIngestionClient.Tenant,
                    productId),
                requestContent: null).Result;
            Console.WriteLine(product.ToString());

            // Try getting a submission, and print its contents, just for information
            if (product.isCommitted?.Value)
            {
                Console.WriteLine("Getting a submission");

                dynamic submission = client.Invoke<dynamic>(
                    HttpMethod.Get,
                    relativeUrl: string.Format(
                        CultureInfo.InvariantCulture,
                        HardwareIngestionClient.GetSubmissionUrlTemplate,
                        HardwareIngestionClient.Version,
                        HardwareIngestionClient.Tenant,
                        productId,
                        submissionId),
                    requestContent: null).Result;

                Console.WriteLine(submission.ToString());
            }


            // Create a new product.
            Console.WriteLine("Creating a new product");
            var sampleProduct = this.GetSampleProduct();
            dynamic newProduct = client.Invoke<dynamic>(
                HttpMethod.Post,
                relativeUrl: string.Format(
                    CultureInfo.InvariantCulture,
                    HardwareIngestionClient.ProductUrlTemplate,
                    HardwareIngestionClient.Version,
                    HardwareIngestionClient.Tenant),
                requestContent: sampleProduct).Result;

            Console.WriteLine("Newly created product.......\n\n");
            Console.WriteLine(newProduct.ToString());

            Console.WriteLine("Creating a new Submission");
            var sampleSubmission = new Submission { Name = "VST_vtest1_nic_6_init", Type = SubmissionTypes.Initial };

            dynamic newSubmission = client.Invoke<dynamic>(
                HttpMethod.Post,
                relativeUrl: string.Format(
                    CultureInfo.InvariantCulture,
                    HardwareIngestionClient.CreateSubmissionUrlTemplate,
                    HardwareIngestionClient.Version,
                    HardwareIngestionClient.Tenant,
                    newProduct.id),
                requestContent: sampleSubmission).Result;

            Console.WriteLine("Newly created submission.......\n\n");
            Console.WriteLine(newSubmission.ToString());

            // Upload a package with to the SAS url returned with the submission
            var fileUploadUrl = newSubmission.downloads.items[0].url.Value as string;
            Console.WriteLine("FileUploadUrl: " + fileUploadUrl);
            Console.WriteLine("Uploading file");
            //TODO: Please change to upload your file
            HardwareIngestionClient.UploadFileToBlobAsync(@"E:\Work\HDC\HalCab.cab", fileUploadUrl).Wait();

            // Tell the system that we are done updating the submission.
            // Update the submission
            Console.WriteLine("Committing the submission");
            client.Invoke<dynamic>(
                HttpMethod.Post,
                relativeUrl: string.Format(
                    CultureInfo.InvariantCulture,
                    HardwareIngestionClient.CommitSubmissionUrlTemplate,
                    HardwareIngestionClient.Version,
                    HardwareIngestionClient.Tenant,
                    newProduct.id,
                    newSubmission.id),
                requestContent: null).Wait();

            // Let's periodically check the status until it changes from "CommitsStarted" to either
            // successful status or a failure.
            Console.WriteLine("Waiting for the submission commit processing to complete. This may take a couple of minutes.");
            string submissionStatus = null;
            do
            {
                Task.Delay(TimeSpan.FromSeconds(5)).Wait();
                dynamic statusResource = client.Invoke<dynamic>(
                    HttpMethod.Get,
                    relativeUrl: string.Format(
                        CultureInfo.InvariantCulture,
                        HardwareIngestionClient.ProductSubmissionStatusUrlTemplate,
                        HardwareIngestionClient.Version,
                        HardwareIngestionClient.Tenant,
                        newProduct.id,
                        newSubmission.id),
                    requestContent: null).Result;

                submissionStatus = statusResource.commitStatus.Value as string;
                Console.WriteLine("Current status: " + submissionStatus);
            }
            while ("commitReceived".Equals(submissionStatus));

            if ("commitFailed".Equals(submissionStatus))
            {
                Console.WriteLine("Submission has failed. Please check errors in the statusResource response.");
                return;
            }
            else
            {
                Console.WriteLine("Submission commit success! Here are some data:");
                dynamic submission = client.Invoke<dynamic>(
                    HttpMethod.Get,
                    relativeUrl: string.Format(
                        CultureInfo.InvariantCulture,
                        HardwareIngestionClient.GetSubmissionUrlTemplate,
                        HardwareIngestionClient.Version,
                        HardwareIngestionClient.Tenant,
                        newProduct.id,
                        newSubmission.id),
                    requestContent: null).Result;
                Console.WriteLine("Newly created Submission...:\n\n ");
                Console.WriteLine(submission.ToString());
            }
        }

        private Product GetSampleProduct()
        {
            var product = new Product
            {
                ProductName = "Test_Network_Product2-R",
                TestHarness = TestHarness.Attestation,
                AnnouncementDate = DateTime.UtcNow,
                DeviceMetadataCategory = "Network.NIC.Ethernet",
                SelectedProductTypes = new Dictionary<string, string>
                {
                    //{ "Windows_v100_TH2", "Network Media Device" },
                    { "Windows_v100_RS1", "Network Media Device" },
                    { "Windows_v100Server_RS1", "Network Media Device" },
                },
                DeviceMetadataIds = new List<Guid>
                {
                    Guid.NewGuid(),
                    Guid.NewGuid()
                },
                FirmwareVersion = "3.2.0.1022",
                DeviceType = DeviceType.External,
                IsTestSign = false,                
                MarketingNames = new List<string> { "Contoso® Rapid Transfer Card Enterprise" },
                RequestedSignatures = new List<string>
                {
                    "WINDOWS_v100_TH2_FULL",
                    "WINDOWS_v100_X64_TH2_FULL",
                    "WINDOWS_v100_RS1_FULL",
                    "WINDOWS_v100_X64_RS1_FULL",
                    "WINDOWS_v100_RS2_FULL",
                    "WINDOWS_v100_X64_RS2_FULL",
                    "WINDOWS_v100_RS3_FULL",
                    "WINDOWS_v100_X64_RS3_FULL"
                },
                AdditionalAttributes = new AdditionalAttributes(),
            };

            return product;
        }
    }
}
