﻿// Sample Code is provided for the purpose of illustration only and is not intended to be used in a production environment. 
// THIS SAMPLE CODE AND ANY RELATED INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
// EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY 
// AND/OR FITNESS FOR A PARTICULAR PURPOSE

namespace HardwareApiCSharpSample
{
    using System;
    using System.Collections.Generic;
    using System.Globalization;
    using System.Net.Http;
    using System.Threading.Tasks;
    using HardwareApiCSharpSample.Models;
    using HardwareApiCSharpSample.Models.ShippingLabels;

    /// <summary>
    /// This sample creates shipping labels for existing product & submission
    /// There are three types - 
    ///     shipping label for windows update
    ///     shipping label for sharing with a partner
    /// </summary>
    public class ShippingLabelSamples
    {
        private ClientConfiguration _clientConfiguration;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="c">An instance of ClientConfiguration that contains all parameters populated</param>
        public ShippingLabelSamples(ClientConfiguration clientConfiguration)
        {
            this._clientConfiguration = clientConfiguration;
        }

        public void RunShippingLabelSamples()
        {
            // **********************
            //       SETTINGS
            // **********************
            var productId = this._clientConfiguration.ProductId;
            var submissionId = this._clientConfiguration.SubmissionId;
            var shippingLabelId = this._clientConfiguration.ShippingLabelId;

            var clientId = this._clientConfiguration.ClientId;
            var clientSecret = this._clientConfiguration.ClientSecret;
            var serviceEndpoint = this._clientConfiguration.ServiceUrl;
            var tokenEndpoint = this._clientConfiguration.TokenEndpoint;

            // Get authorization token
            Console.WriteLine("Getting authorization token ");
            var accessToken = HardwareIngestionClient.GetClientCredentialAccessTokenAsync(
                tokenEndpoint,
                clientId,
                clientSecret).Result;

            // create ingestion client
            var client = new HardwareIngestionClient(accessToken, serviceEndpoint);

            // GET Shipping Label
            this.GetShippingLabelDetails(productId, submissionId, shippingLabelId, client);

            // CREATE "Publish to Windows" Shipping Label
            this.CreateShippingLabelForWU(productId, submissionId, client);

            // CREATE "Share with Partner" Shipping Label
            this.CreateShippingLabelForShare(productId, submissionId, client);

            // PATCH / PARTIAL Update Shipping Label
            this.PatchShippingLabel(productId, submissionId, shippingLabelId, client);
        }
        
        private void PatchShippingLabel(string productId, string submissionId, string shippingLabelId, HardwareIngestionClient client)
        {
            var httpMethodPatch = new HttpMethod("PATCH");

            // Create a new shipping label. SHARE
            Console.WriteLine("Update (patch) an existing shipping label");
            var sampleShippingLabelShare = this.GetSampleShippingLabelForUpdate();
            dynamic newShippingLabelShare = client.Invoke<dynamic>(
                httpMethodPatch,
                relativeUrl: string.Format(
                    CultureInfo.InvariantCulture,
                    HardwareIngestionClient.UpdateShippingLabelUrlTemplate,
                    HardwareIngestionClient.Version,
                    HardwareIngestionClient.Tenant,
                    productId,
                    submissionId,
                    shippingLabelId),
                requestContent: sampleShippingLabelShare).Result;

            Console.WriteLine("Shippinglabel after patch operation initiated.......\n\n");
            Console.WriteLine("Since shipping label patch returns NoContent-result (HttpStatus 204), the newShippingLabelShare object would be null");
        }
        
        private void CreateShippingLabelForShare(string productId, string submissionId, HardwareIngestionClient client)
        {
            // Create a new shipping label. SHARE
            Console.WriteLine("Creating a new shipping label share with partner");
            var sampleShippingLabelShare = this.GetSampleShippingLabelForShare();
            dynamic newShippingLabelShare = client.Invoke<dynamic>(
                HttpMethod.Post,
                relativeUrl: string.Format(
                    CultureInfo.InvariantCulture,
                    HardwareIngestionClient.CreateShippingLabelUrlTemplate,
                    HardwareIngestionClient.Version,
                    HardwareIngestionClient.Tenant,
                    productId,
                    submissionId),
                requestContent: sampleShippingLabelShare).Result;

            Console.WriteLine("Newly created shared-shippinglabel.......\n\n");
            Console.WriteLine(newShippingLabelShare.ToString());
        }

        private void CreateShippingLabelForWU(string productId, string submissionId, HardwareIngestionClient client)
        {
            // Create a new shipping label. WU
            Console.WriteLine("Creating a new shipping label for windows");
            var sampleShippingLabelWu = this.GetSampleShippingLabelForWu();
            dynamic newShippingLabelWu = client.Invoke<dynamic>(
                HttpMethod.Post,
                relativeUrl: string.Format(
                    CultureInfo.InvariantCulture,
                    HardwareIngestionClient.CreateShippingLabelUrlTemplate,
                    HardwareIngestionClient.Version,
                    HardwareIngestionClient.Tenant,
                    productId,
                    submissionId),
                requestContent: sampleShippingLabelWu).Result;

            Console.WriteLine("Newly created WU-shippinglabel.......\n\n");
            Console.WriteLine(newShippingLabelWu.ToString());
        }

        private void GetShippingLabelDetails(string productId, string submissionId, string shippingLabelId, HardwareIngestionClient client)
        {
            // Getting a specific shipping label here,
            // Though one can get all shipping labels under a product/submission use GetAllShippingLabelsUrlTemplate
            Console.WriteLine("Getting a shipping label by Id");
            dynamic shippingLabel = client.Invoke<dynamic>(
                HttpMethod.Get,
                relativeUrl: string.Format(
                    CultureInfo.InvariantCulture,
                    HardwareIngestionClient.GetShippingLabelUrlTemplate,
                    HardwareIngestionClient.Version,
                    HardwareIngestionClient.Tenant,
                    productId,
                    submissionId,
                    shippingLabelId),
                requestContent: null).Result;
            Console.WriteLine(shippingLabel.ToString());
        }

        private ShippingLabelForCreation GetSampleShippingLabelForWu()
        {
            var shippingLabelWu = new ShippingLabelForCreation()
            {
                Name = "Test_SL_Wu",
                Destination = ShippingLabelDestination.WindowsUpdate,
                Targeting = this.GetTargetingInfoForCreation(),

                // Note: since this is a "Publish to Windows" Shipping Label - RecipientSpecification is not being set
                PublishingSpecifications = this.GetPublishingSpecification(),
                
            };

            return shippingLabelWu;
        }

        private ShippingLabelForCreation GetSampleShippingLabelForShare()
        {
            var shippingLabelShare = new ShippingLabelForCreation()
            {
                Name = "Test_SL_Share",
                Destination = ShippingLabelDestination.AnotherPartner,
                Targeting = this.GetTargetingInfoForCreation(true),

                // Note: since this is a "Share with Partner" Shipping Label - PublishingSpecification is not being set
                RecipientSpecifications = this.GetRecipientSpecifications(),
            };

            return shippingLabelShare;
        }

        private RecipientSpecifications GetRecipientSpecifications()
        {
            var recipientSpecifications = new RecipientSpecifications()
            {
                // shipping label cannot be shared twice 
                // you may get exception when you use same receiverid twice under a submission
                ReceiverPublisherId = "1234568", 
                EnforceChidTargeting = false
            };

            return recipientSpecifications;
        }

        private PublishingSpecifications GetPublishingSpecification()
        {
            var publishingSpecifications = new PublishingSpecifications()
            {
                GoLiveDate = DateTimeOffset.Now,
                IsDisclosureRestricted = false,
                PublishToWindows10s = false,
                IsAutoInstallDuringOSUpgrade = false,
                IsAutoInstallOnApplicableSystems = false,
                VisibleToAccounts = null,

                AdditionalInfoForMsApproval = new AdditionalInfoForMSApproval()
                {
                    AffectedOems = null,
                    BusinessJustification = "provide your business justification",
                    HasUiSoftware = false,
                    IsCoEngineered = false,
                    // TODO: Stop using this property.
                    // IsEmergencyPublication = false,
                    IsForUnreleasedHardware = false,
                    IsRebootRequired = false,
                    MicrosoftContact = "someone@microsoft.com",
                    ValidationsPerformed = null
                }
            };

            return publishingSpecifications;
        }

        private ShippingLabelTargetingInfo GetTargetingInfoForCreation(bool isForShare = false)
        {
            // NOTE: Refer to the Metadata sample to know how to download driverMetadata-file
            // One has to refer to Driver-Metadata file to create appropraite targetinginfo

            var targeting = new ShippingLabelTargetingInfo()
            {
                Chids = new List<DistributionChid>()
                    {
                        new DistributionChid()
                        {
                            Chid = Guid.NewGuid(), // refer driver-metadata file to assign appropriate value
                            DistributionState = ShippingLabelDistributionState.PendingAdd
                        }
                    },
                HardwareIds = new List<HardwareId>()
                    {
                        // refer driver-metadata file to assign appropriate value to all of these properties
                        new HardwareId()
                        {
                            BundleId = "ef2297fc-7494-45d3-b39b-03685bbb465a",
                            InfId = "toaster.inf",
                            OperatingSystemCode = "WINDOWS_v100_RS2_FULL",
                            PnpString = @"{0d7ccb1e-21e4-4a2c-9dd4-167b267bc4d2}\mstoaster",
                            DistributionState = ShippingLabelDistributionState.PendingAdd
                        }
                    },
                RestrictedToAudiences = null , // Use audience api to get the audience list - refer AudienceSample.cs

            };

            if (!isForShare)
            {
                /*
                    // TODO: If you want to provide co-engineering flooring-ceiling
                    CoEngDriverPublishInfo = new CoEngPublishRestriction()
                    {
                        CeilingBuildNumber = 123456,
                        FlooringBuildNumber = 123455
                    },
                */
                // One can set either CoEng or InService flooring-ceiling
                targeting.InServicePublishInfo = new InServicePublishRestriction()
                {
                    Ceiling = RestrictionOS.RS3,
                    //Flooring = RestrictionOS.RS1
                };
            }

            return targeting;
        }

        private ShippingLabelForUpdation GetSampleShippingLabelForUpdate(bool isForShare = false)
        {
            // NOTE: Refer to the Metadata sample to know how to download driverMetadata-file
            // One has to refer to Driver-Metadata file to create appropraite targettinginfo

            var shippingLabelForUpdate = new ShippingLabelForUpdation()
            {
                // TODO: Please note we have unified the name here - use Targeting instead of TargetingInfo
                Targeting = new ShippingLabelTargetingInfoForUpdation()
                {
                    Chids = new List<DistributionChidForUpdation>()
                    {
                        new DistributionChidForUpdation()
                        {
                            Chid = Guid.NewGuid(),  // refer driver-metadata file to assign appropriate value
                            Action = ShippingLabelDistributionAction.Add
                        }
                    },
                    HardwareIds = new List<HardwareIdForUpdation>()
                    {
                        // refer driver-metadata file to assign appropriate value to all of these properties
                        new HardwareIdForUpdation()
                        {
                            BundleId = "ef2297fc-7494-45d3-b39b-03685bbb465a",
                            InfId = "toaster.inf",
                            OperatingSystemCode = "WINDOWS_v100_X64_RS2_FULL",
                            PnpString = @"{0d7ccb1e-21e4-4a2c-9dd4-167b267bc4d2}\mstoaster",
                            Action = ShippingLabelDistributionAction.Add
                        }
                    },
                    RestrictedToAudiences = null,  // Use audience api to get the audience list - refer AudienceSample.cs
                },
                BusinessJustification = "some business justification"
            };

            if (!isForShare)
            {
                /*
                    // TODO: If you want to provide co-engineering flooring-ceiling
                    CoEngDriverPublishInfo = new CoEngPublishRestriction()
                    {
                        CeilingBuildNumber = 123456,
                        FlooringBuildNumber = 123455
                    },
                */
                // One can set either CoEng or InService flooring-ceiling
                shippingLabelForUpdate.Targeting.InServicePublishInfo = new InServicePublishRestriction()
                {
                    Ceiling = RestrictionOS.RS3,
                    //Flooring = RestrictionOS.RS1
                };
            }

            return shippingLabelForUpdate;
        }
    }
}
